"""
Jinja2 environment and extensions loading.

Source: https://github.com/cookiecutter/cookiecutter
"""
import sys

from jinja2 import Environment, StrictUndefined
from jinja2.nativetypes import NativeEnvironment


class ExtensionLoaderMixin:
    """
    Mixin providing sane loading of extensions specified in a given context.
    """

    def __init__(self, *_args, context=None, path=None, **kwargs):
        """
        Initialize the Jinja2 Environment object while loading extensions.
        """
        extensions = [
            "mk_scaffold.extensions.JsonifyExtension",
            "mk_scaffold.extensions.RandomStringExtension",
            "mk_scaffold.extensions.SlugifyExtension",
            "mk_scaffold.extensions.StringExtension",
            "jinjazzle.ext.UUIDExtension",
            "jinja2_time.TimeExtension",
        ]

        extensions += context or []

        # Add template path to sys.path for loading
        if path is not None:
            sys.path.append(path)

        try:
            super().__init__(extensions=extensions, **kwargs)
        except ModuleNotFoundError as err:
            sys.exit(f'error: failed to load extensions from "{path}" directory: {err}')


class StrictEnvironment(ExtensionLoaderMixin, Environment):
    """
    Create strict Jinja2 environment.

    Jinja2 environment will raise error on undefined variable in template-
    rendering context.
    """

    def __init__(self, *_args, **kwargs):
        """
        Set the standard Cookiecutter StrictEnvironment.

        Also loading extensions defined in mk-scaffold 'extensions' key.
        """
        super().__init__(**kwargs)
        self.undefined = StrictUndefined


class StrictNativeEnvironment(ExtensionLoaderMixin, NativeEnvironment):
    """
    Create strict Jinja2 environment.

    Jinja2 environment will raise error on undefined variable in template-
    rendering context.
    """

    def __init__(self, *_args, **kwargs):
        """
        Set the standard Cookiecutter StrictEnvironment.

        Also loading extensions defined in mk-scaffold 'extensions' key.
        """
        super().__init__(**kwargs)
        self.undefined = StrictUndefined



import copy
import sys

import requests
import yaml


def merge(data):
    includes = data.get("include")
    if not includes:
        return data

    for include in includes:
        if include.startswith("http://") or include.startswith("https://"):
            result = requests.get(include, timeout=10)
            result.raise_for_status()
            include_data = yaml.safe_load(result.text)

            # Source is what we have in our file
            # Destination is what we just loaded and will replace previous
            # questions
            src_questions = copy.deepcopy(data["questions"])
            dst_questions = include_data["questions"]

            src_questions = {x["name"]: x for x in src_questions}
            dst_questions = {x["name"]: x for x in dst_questions}

            dst_questions = {**dst_questions, **src_questions}
            data["questions"] = [v for k, v in dst_questions.items()]

        else:
            sys.exit("error: inclusion method not implemented")
    return data



import sys

import yaml


def load(args, data):
    """
    Load an answers file
    """
    file = args.get("input_file")
    if not file:
        return data

    try:
        with open(file, encoding="UTF-8") as fd:
            answers = yaml.safe_load(fd)
    except Exception as err:
        sys.exit(f"error: failed to open '{file}': {err}")

    # Copy answers without emptying out values that could have been present
    # in questions file.
    if "answers" not in data:
        data["answers"] = {}
    if not answers:
        return data
    for k, v in answers.get("answers").items():
        data["answers"][k] = v
    return data
